#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>

#include"ds1620_134.h"

void init1620(void)
{
	uint8_t cur_config;
	/*
	Alles auf Ausgang, CLK setzen, andere loeschen
	- ist auch der Zustand, wenn keine der "get/put1620byte"
	funktionen aufgerufen ist. 
	*/
	DS1620DDR |= ((1<<DS1620CLK)|(1<<DS1620DQ)|(1<<DS1620RST));
	DS1620PORT |= (1<<DS1620CLK);
	DS1620PORT &= ~((1<<DS1620DQ)|(1<<DS1620RST));
	
	_delay_us(1);
	SETRST;
	/*Configregister auslesen*/
	put1620byte(0xAC);
	cur_config = get1620byte();
	CLEARRST;
	
	/* Auf Bit 3 testen, das ist fest auf "1" */
	if((cur_config & 0x18) != 0x08 ){
		/* ERROR:
		Tu was, der Sensor antwortet nicht!!
		*/
	}
	/* check mode settings  */
	if ( (cur_config & ((1<<CPU)|(1<<ONESHOT))) != 0x03 )                   
	{
		SETRST;
		/* set to CPU & 1SHOT mode */
		put1620byte(0x0C);
		put1620byte( (1<<CPU)|(1<<ONESHOT) );
		CLEARRST;
	}
	/*ist so aus der Appnote uebernommen, wird aber nicht im Datenblatt erwaehnt*/
	_delay_ms(10);
	SETRST;
	put1620byte(0x22);
	CLEARRST;
}
/* ein Byte vom Sensor lesen*/
uint8_t  get1620byte( void )
{
	/*	bits - schleifenzhler, entspricht der Nr. des aktuellen Bits
		read - gelesenes Byte
		mask - Byte, in dem das aktuelle bit gesetzt ist*/
	uint8_t bits, data=0, mask=1;
	
	MAKEDQINPUT;
	for (bits=0; bits<8; bits++){
		CLEARCLK;
		_delay_us(1);
		
		/* bitsetzen, wenn Pin = 1  */
		if (DS1620PIN & (1<<DS1620DQ))
			data |= mask;
			
		SETCLK;
		_delay_us(1);
		/*maske schieben fr das nchste Bit*/
		mask <<= 1;
		}
	MAKEDQOUTPUT;
	return data;
	}

void put1620byte(uint8_t data)
{
	uint8_t bits, mask=1;
	SETRST;
	for ( bits=0; bits<8; bits++){
		CLEARCLK;
		_delay_us(1);
		/*wenn da eine 1 ist, DQ setzen, sonst sicher loeschen*/
		if (data & mask)
			DS1620PORT |= (1<<DS1620DQ);
		else
			DS1620PORT &= (1<<DS1620DQ);
			
		SETCLK;
		_delay_us(1);
		
		mask=(mask<<1);
	}
	// DQ wieder auf 'low'
	DS1620PORT &= ~(1<<DS1620DQ);
	return;
}

/*diese Funktion ist selbst geschrieben*/
int16_t get1620temp(void)
{	
	uint8_t tmp, cur_config;
	int16_t temperature;
/* Das Configregister abfragen, bis "DONE" gesetzt wird*/
	do{
		SETRST;
		put1620byte(0xAC);	/* open status register */
		cur_config = get1620byte();	/* get status byte */
		CLEARRST;
	}while ( (cur_config & (1<<DONE)) != (1<<DONE) );
	
	SETRST;
	put1620byte(0xA0);
	tmp = get1620byte();
	temperature = tmp;
	tmp = get1620byte();
	CLEARRST;
	if(tmp==1);
		tmp |= 0xFE;
		
	temperature += tmp*256;
	return temperature;
}
